@@ -44,10 +44,15 @@ typedef struct
 
   int okay;
 
-  /* A flag telling that the a decryption failed and an optional error
-   * code to further specify the failure.  */
+  /* Indicates that list only mode is active.  */
+  int list_only;
+
+  /* A flag telling that the decryption failed and two optional error
+   * codes to further specify the failure for public key decryption and
+   * symmetric decryption.  */
   int failed;
   gpg_error_t pkdecrypt_failed;
+  gpg_error_t symdecrypt_failed;
 
   /* At least one secret key is not available.  gpg issues NO_SECKEY
    * status lines for each key the message has been encrypted to but
@@ -228,7 +233,22 @@ parse_status_error (char *args, op_data_t opd)
       opd->result.legacy_cipher_nomdc = 1;
       opd->not_integrity_protected = 1;
     }
+  else if (!strcmp (field[0], "symkey_decrypt.maybe_error"))
+    {
+      switch (gpg_err_code (err))
+        {
+        case GPG_ERR_BAD_PASSPHRASE:
+          /* A bad passphrase is severe enough that we return this
+           * error code.  */
+          opd->symdecrypt_failed = err;
+          break;
 
+        default:
+          /* For now all other error codes are ignored and the
+           * standard DECRYPT_FAILED is returned.  */
+          break;
+        }
+    }
   /* Record the first error code.  */
   if (err && !opd->first_status_error)
     opd->first_status_error = err;
@@ -360,7 +380,9 @@ _gpgme_decrypt_status_handler (void *priv, gpgme_status_code_t code,
   switch (code)
     {
     case GPGME_STATUS_FAILURE:
-      opd->failure_code = _gpgme_parse_failure (args);
+      if (!opd->failure_code
+          || gpg_err_code (opd->failure_code) == GPG_ERR_GENERAL)
+        opd->failure_code = _gpgme_parse_failure (args);
       break;
 
     case GPGME_STATUS_EOF:
@@ -376,6 +398,8 @@ _gpgme_decrypt_status_handler (void *priv, gpgme_status_code_t code,
           /* This comes from a specialized ERROR status line.  */
           if (opd->pkdecrypt_failed)
             return opd->pkdecrypt_failed;
+          if (opd->symdecrypt_failed)
+            return opd->symdecrypt_failed;
 
           /* For an integrity failure return just DECRYPTION_FAILED;
            * the actual cause can be taken from an already set
@@ -398,7 +422,7 @@ _gpgme_decrypt_status_handler (void *priv, gpgme_status_code_t code,
           /* Generic decryption failed error code.  */
           return gpg_error (GPG_ERR_DECRYPT_FAILED);
         }
-      else if (!opd->okay)
+      else if (!opd->okay && !opd->list_only)
         {
           /* No data was found.  */
           return gpg_error (GPG_ERR_NO_DATA);
@@ -519,7 +543,8 @@ decrypt_status_handler (void *priv, gpgme_status_code_t code, char *args)
 
 
 gpgme_error_t
-_gpgme_op_decrypt_init_result (gpgme_ctx_t ctx, gpgme_data_t plaintext)
+_gpgme_op_decrypt_init_result (gpgme_ctx_t ctx, gpgme_data_t plaintext,
+                               gpgme_decrypt_flags_t flags)
 {
   gpgme_error_t err;
   void *hook;
@@ -531,6 +556,7 @@ _gpgme_op_decrypt_init_result (gpgme_ctx_t ctx, gpgme_data_t plaintext)
   if (err)
     return err;
 
+  opd->list_only = !!(flags & GPGME_DECRYPT_LISTONLY);
   opd->last_recipient_p = &opd->result.recipients;
   opd->plaintext_dserial = _gpgme_data_get_dserial (plaintext);
   return 0;
@@ -550,7 +576,7 @@ _gpgme_decrypt_start (gpgme_ctx_t ctx, int synchronous,
   if (err)
     return err;
 
-  err = _gpgme_op_decrypt_init_result (ctx, plain);
+  err = _gpgme_op_decrypt_init_result (ctx, plain, flags);
   if (err)
     return err;
 
