@@ -69,6 +69,7 @@ gpgme_error_t _gpgme_engine_set_locale (engine_t engine, int category,
 					const char *value);
 gpgme_error_t _gpgme_engine_set_protocol (engine_t engine,
 					  gpgme_protocol_t protocol);
+void _gpgme_engine_set_engine_flags (engine_t engine, gpgme_ctx_t ctx);
 void _gpgme_engine_release (engine_t engine);
 void _gpgme_engine_set_status_cb (engine_t engine,
                                   gpgme_status_cb_t cb, void *cb_value);
@@ -77,8 +78,7 @@ void _gpgme_engine_set_status_handler (engine_t engine,
 				       void *fnc_value);
 gpgme_error_t _gpgme_engine_set_command_handler (engine_t engine,
 						 engine_command_handler_t fnc,
-						 void *fnc_value,
-						 gpgme_data_t data);
+						 void *fnc_value);
 gpgme_error_t
 _gpgme_engine_set_colon_line_handler (engine_t engine,
 				      engine_colon_line_handler_t fnc,
@@ -97,11 +97,13 @@ gpgme_error_t _gpgme_engine_op_edit (engine_t engine, int type,
 				     gpgme_ctx_t ctx /* FIXME */);
 gpgme_error_t _gpgme_engine_op_encrypt (engine_t engine,
 					gpgme_key_t recp[],
+                                        const char *recpstring,
 					gpgme_encrypt_flags_t flags,
 					gpgme_data_t plain, gpgme_data_t ciph,
 					int use_armor);
 gpgme_error_t _gpgme_engine_op_encrypt_sign (engine_t engine,
 					     gpgme_key_t recp[],
+                                             const char *recpstring,
 					     gpgme_encrypt_flags_t flags,
 					     gpgme_data_t plain,
 					     gpgme_data_t ciph,
@@ -129,33 +131,43 @@ gpgme_error_t _gpgme_engine_op_keysign (engine_t engine,
                                         unsigned long expires,
                                         unsigned int flags,
                                         gpgme_ctx_t ctx);
+gpgme_error_t _gpgme_engine_op_revsig (engine_t engine,
+                                       gpgme_key_t key,
+                                       gpgme_key_t signing_key,
+                                       const char *userid,
+                                       unsigned int flags);
 gpgme_error_t _gpgme_engine_op_tofu_policy (engine_t engine,
                                             gpgme_key_t key,
                                             gpgme_tofu_policy_t policy);
 gpgme_error_t _gpgme_engine_op_import (engine_t engine,
 				       gpgme_data_t keydata,
-                                       gpgme_key_t *keyarray);
+                                       gpgme_key_t *keyarray,
+                                       const char *keyids[],
+                                       const char *import_filter,
+                                       const char *import_options,
+                                       const char *key_origin);
 gpgme_error_t _gpgme_engine_op_keylist (engine_t engine,
 					const char *pattern,
 					int secret_only,
-					gpgme_keylist_mode_t mode,
-					int engine_flags);
+					gpgme_keylist_mode_t mode);
 gpgme_error_t _gpgme_engine_op_keylist_ext (engine_t engine,
 					    const char *pattern[],
 					    int secret_only,
 					    int reserved,
-					    gpgme_keylist_mode_t mode,
-					    int engine_flags);
+					    gpgme_keylist_mode_t mode);
 gpgme_error_t _gpgme_engine_op_keylist_data (engine_t engine,
+					     gpgme_keylist_mode_t mode,
 					     gpgme_data_t data);
 gpgme_error_t _gpgme_engine_op_sign (engine_t engine, gpgme_data_t in,
-				     gpgme_data_t out, gpgme_sig_mode_t mode,
+				     gpgme_data_t out, gpgme_sig_mode_t flags,
 				     int use_armor, int use_textmode,
 				     int include_certs,
 				     gpgme_ctx_t ctx /* FIXME */);
 gpgme_error_t _gpgme_engine_op_trustlist (engine_t engine,
 					  const char *pattern);
-gpgme_error_t _gpgme_engine_op_verify (engine_t engine, gpgme_data_t sig,
+gpgme_error_t _gpgme_engine_op_verify (engine_t engine,
+                                       gpgme_verify_flags_t flags,
+                                       gpgme_data_t sig,
 				       gpgme_data_t signed_text,
 				       gpgme_data_t plaintext,
                                        gpgme_ctx_t ctx);
@@ -173,6 +185,10 @@ gpgme_error_t _gpgme_engine_op_assuan_transact
                  gpgme_assuan_status_cb_t status_cb,
                  void *status_cb_value);
 
+gpgme_error_t _gpgme_engine_op_getdirect (engine_t engine, const char *argv[],
+                                          gpgme_data_t dataout,
+                                          unsigned int flags);
+
 gpgme_error_t _gpgme_engine_op_conf_load (engine_t engine,
 					  gpgme_conf_comp_t *conf_p);
 gpgme_error_t _gpgme_engine_op_conf_save (engine_t engine,
@@ -208,9 +224,13 @@ gpgme_error_t _gpgme_engine_op_spawn (engine_t engine,
                                       gpgme_data_t dataout,
                                       gpgme_data_t dataerr,
                                       unsigned int flags);
-
-/* The available engine option flags.  */
-#define GPGME_ENGINE_FLAG_OFFLINE        1
-
+gpgme_error_t _gpgme_engine_op_setexpire (engine_t engine,
+                                          gpgme_key_t key,
+                                          unsigned long expires,
+                                          const char *subfprs,
+                                          unsigned int reserved);
+gpgme_error_t _gpgme_engine_op_setownertrust (engine_t engine,
+                                              gpgme_key_t key,
+                                              const char *value);
 
 #endif /* ENGINE_H */
